// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

// THIS FILE IS AUTO-GENERATED, PLEASE DO NOT MODIFY BY HAND.

part of molview_messages.v0;

/// DataServer messages
/// ===================
/// The DataServer is the central, in-memory data storage in MolView. The
/// DataServer manages the retrieval of data records from external databases and
/// web-services. It can also be used to integrate local resources with external
/// resources. Modules can submit new data tables to the DataServer. The
/// DataServer takes care of interconnecting tables and finding the best way to
/// get the data that is requested by modules. To do so the DataServer stores a
/// map of keys that link to the most recent data in the application. These keys
/// can be used to retrieve related data.

/// Activate a new resource table in the DataServer
class ActivateTable extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_messages.v0.ActivateTable';

  /// Table information
  final DataTable table;

  /// Constructor
  ActivateTable(final DataTable table)
      : table = table,
        super(id, {'table': table});

  /// Internal constructor for fromJson
  ActivateTable._from(final Map<String, dynamic> data, DataTable table)
      : table = table,
        super(id, data);

  /// Data map input constructor
  factory ActivateTable.fromJson(final Map<String, dynamic> data) {
    DataTable table;

    if (data.containsKey('table') && data['table'] is Map<String, dynamic>) {
      table = new DataTable.fromJson(data['table'] as Map<String, dynamic>);
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'table'");
    }

    return new ActivateTable._from(data, table);
  }
}

/// A table activation was rejected
class TableRejected extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_messages.v0.TableRejected';

  /// Rejection reason
  final TableRejectionReason reason;

  /// User friendly rejection message
  final String message;

  /// Constructor
  TableRejected(final TableRejectionReason reason, final String message)
      : reason = reason,
        message = message,
        super.unchecked(id, {'reason': reason.index, 'message': message});

  /// Internal constructor for fromJson
  TableRejected._from(final Map<String, dynamic> data,
      TableRejectionReason reason, String message)
      : reason = reason,
        message = message,
        super.unchecked(id, data);

  /// Data map input constructor
  factory TableRejected.fromJson(final Map<String, dynamic> data) {
    TableRejectionReason reason;
    String message;

    if (data.containsKey('reason') && data['reason'] is int) {
      reason = TableRejectionReason.values[data['reason'] as int];
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'reason'");
    }

    if (data.containsKey('message') && data['message'] is String) {
      message = data['message'] as String;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'message'");
    }

    return new TableRejected._from(data, reason, message);
  }
}

/// A new table was registered
class TableRegistered extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_messages.v0.TableRegistered';

  /// Resource table details
  final DataTable table;

  /// Constructor
  TableRegistered(final DataTable table)
      : table = table,
        super(id, {'table': table});

  /// Internal constructor for fromJson
  TableRegistered._from(final Map<String, dynamic> data, DataTable table)
      : table = table,
        super(id, data);

  /// Data map input constructor
  factory TableRegistered.fromJson(final Map<String, dynamic> data) {
    DataTable table;

    if (data.containsKey('table') && data['table'] is Map<String, dynamic>) {
      table = new DataTable.fromJson(data['table'] as Map<String, dynamic>);
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'table'");
    }

    return new TableRegistered._from(data, table);
  }
}

/// Request record from a table
class RetrieveData extends OmnibusRequest {
  /// omnibusId
  static const id = 'molview_messages.v0.RetrieveData';

  /// Target database
  final String database;

  /// Target table
  final String table;

  /// Input key data class
  final String inkeyClass;

  /// Input key value
  final String inkeyValue;

  /// Retrieval parameters
  final Map<String, String> parameters;

  /// Requested keys by their index
  final List<int> keys;

  /// Requested values by their index
  final List<int> values;

  /// Constructor
  RetrieveData(final String database, final String table,
      final String inkeyClass, final String inkeyValue,
      {final Map<String, String> parameters: const {},
      final List<int> keys: const [],
      final List<int> values: const []})
      : database = database,
        table = table,
        inkeyClass = inkeyClass,
        inkeyValue = inkeyValue,
        parameters = parameters,
        keys = keys,
        values = values,
        super.unchecked(id, {
          'database': database,
          'table': table,
          'inkeyClass': inkeyClass,
          'inkeyValue': inkeyValue,
          'parameters': parameters,
          'keys': keys,
          'values': values
        });

  /// Internal constructor for fromJson
  RetrieveData._from(
      final Map<String, dynamic> data,
      String database,
      String table,
      String inkeyClass,
      String inkeyValue,
      Map<String, String> parameters,
      List<int> keys,
      List<int> values)
      : database = database,
        table = table,
        inkeyClass = inkeyClass,
        inkeyValue = inkeyValue,
        parameters = parameters,
        keys = keys,
        values = values,
        super.unchecked(id, data);

  /// Data map input constructor
  factory RetrieveData.fromJson(final Map<String, dynamic> data) {
    String database;
    String table;
    String inkeyClass;
    String inkeyValue;
    Map<String, String> parameters;
    List<int> keys;
    List<int> values;

    if (data.containsKey('database') && data['database'] is String) {
      database = data['database'] as String;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'database'");
    }

    if (data.containsKey('table') && data['table'] is String) {
      table = data['table'] as String;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'table'");
    }

    if (data.containsKey('inkeyClass') && data['inkeyClass'] is String) {
      inkeyClass = data['inkeyClass'] as String;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'inkeyClass'");
    }

    if (data.containsKey('inkeyValue') && data['inkeyValue'] is String) {
      inkeyValue = data['inkeyValue'] as String;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'inkeyValue'");
    }

    if (data.containsKey('parameters') &&
        data['parameters'] is Map<String, String>) {
      parameters = data['parameters'] as Map<String, String>;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'parameters'");
    }

    if (data.containsKey('keys') && data['keys'] is List<int>) {
      keys = data['keys'] as List<int>;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'keys'");
    }

    if (data.containsKey('values') && data['values'] is List<int>) {
      values = data['values'] as List<int>;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'values'");
    }

    return new RetrieveData._from(data, database, table, inkeyClass, inkeyValue,
        parameters, keys, values);
  }
}

/// Return data to the DataSever
///
/// This message contain the data that is retrieved in response to an
/// mRetrieveData request.
class DataRetrieved extends OmnibusResponse {
  /// omnibusId
  static const id = 'molview_messages.v0.DataRetrieved';

  /// Retrieved data records
  final List<DataRecord> records;

  /// Constructor
  DataRetrieved(final List<DataRecord> records)
      : records = records,
        super(id, {'records': records});

  /// Internal constructor for fromJson
  DataRetrieved._from(final Map<String, dynamic> data, List<DataRecord> records)
      : records = records,
        super(id, data);

  /// Data map input constructor
  factory DataRetrieved.fromJson(final Map<String, dynamic> data) {
    List<DataRecord> records;

    if (data.containsKey('records') &&
        data['records'] is List<Map<String, dynamic>>) {
      records = new List<DataRecord>.generate(
          data['records'].length,
          (int i1) => new DataRecord.fromJson(
              data['records'][i1] as Map<String, dynamic>));
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'records'");
    }

    return new DataRetrieved._from(data, records);
  }
}

/// Send data to the DataServer
class SubmitData extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_messages.v0.SubmitData';

  /// Records that are sumbitted
  final List<DataRecord> records;

  /// Constructor
  SubmitData(final List<DataRecord> records)
      : records = records,
        super(id, {'records': records});

  /// Internal constructor for fromJson
  SubmitData._from(final Map<String, dynamic> data, List<DataRecord> records)
      : records = records,
        super(id, data);

  /// Data map input constructor
  factory SubmitData.fromJson(final Map<String, dynamic> data) {
    List<DataRecord> records;

    if (data.containsKey('records') &&
        data['records'] is List<Map<String, dynamic>>) {
      records = new List<DataRecord>.generate(
          data['records'].length,
          (int i1) => new DataRecord.fromJson(
              data['records'][i1] as Map<String, dynamic>));
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'records'");
    }

    return new SubmitData._from(data, records);
  }
}

/// Find data snippets
class FindSnippets extends OmnibusRequest {
  /// omnibusId
  static const id = 'molview_messages.v0.FindSnippets';

  /// Requested data snippets
  final List<SnippetSelector> snippets;

  /// Alternative starting key data class
  final String inkeyClass;

  /// Alternative starting key value
  final String inkeyValue;

  /// Constructor
  FindSnippets(final List<SnippetSelector> snippets,
      {final String inkeyClass: '', final String inkeyValue: ''})
      : snippets = snippets,
        inkeyClass = inkeyClass,
        inkeyValue = inkeyValue,
        super(id, {
          'snippets': snippets,
          'inkeyClass': inkeyClass,
          'inkeyValue': inkeyValue
        });

  /// Internal constructor for fromJson
  FindSnippets._from(final Map<String, dynamic> data,
      List<SnippetSelector> snippets, String inkeyClass, String inkeyValue)
      : snippets = snippets,
        inkeyClass = inkeyClass,
        inkeyValue = inkeyValue,
        super(id, data);

  /// Data map input constructor
  factory FindSnippets.fromJson(final Map<String, dynamic> data) {
    List<SnippetSelector> snippets;
    String inkeyClass;
    String inkeyValue;

    if (data.containsKey('snippets') &&
        data['snippets'] is List<Map<String, dynamic>>) {
      snippets = new List<SnippetSelector>.generate(
          data['snippets'].length,
          (int i1) => new SnippetSelector.fromJson(
              data['snippets'][i1] as Map<String, dynamic>));
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'snippets'");
    }

    if (data.containsKey('inkeyClass') && data['inkeyClass'] is String) {
      inkeyClass = data['inkeyClass'] as String;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'inkeyClass'");
    }

    if (data.containsKey('inkeyValue') && data['inkeyValue'] is String) {
      inkeyValue = data['inkeyValue'] as String;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'inkeyValue'");
    }

    return new FindSnippets._from(data, snippets, inkeyClass, inkeyValue);
  }
}

/// Return snippets to the data consumer.
class SnippetsFound extends OmnibusResponse {
  /// omnibusId
  static const id = 'molview_messages.v0.SnippetsFound';

  /// Resulting snippets
  final List<SnippetData> snippets;

  /// Constructor
  SnippetsFound(final List<SnippetData> snippets)
      : snippets = snippets,
        super(id, {'snippets': snippets});

  /// Internal constructor for fromJson
  SnippetsFound._from(
      final Map<String, dynamic> data, List<SnippetData> snippets)
      : snippets = snippets,
        super(id, data);

  /// Data map input constructor
  factory SnippetsFound.fromJson(final Map<String, dynamic> data) {
    List<SnippetData> snippets;

    if (data.containsKey('snippets') &&
        data['snippets'] is List<Map<String, dynamic>>) {
      snippets = new List<SnippetData>.generate(
          data['snippets'].length,
          (int i1) => new SnippetData.fromJson(
              data['snippets'][i1] as Map<String, dynamic>));
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'snippets'");
    }

    return new SnippetsFound._from(data, snippets);
  }
}

/// Search table using a text
class TextSearch extends OmnibusRequest {
  /// omnibusId
  static const id = 'molview_messages.v0.TextSearch';

  /// Target database
  final String database;

  /// Target table
  final String table;

  /// Search text
  final String text;

  /// Offset for paged search
  final int offset;

  /// Constructor
  TextSearch(final String database, final String table, final String text,
      final int offset)
      : database = database,
        table = table,
        text = text,
        offset = offset,
        super.unchecked(id, {
          'database': database,
          'table': table,
          'text': text,
          'offset': offset
        });

  /// Internal constructor for fromJson
  TextSearch._from(final Map<String, dynamic> data, String database,
      String table, String text, int offset)
      : database = database,
        table = table,
        text = text,
        offset = offset,
        super.unchecked(id, data);

  /// Data map input constructor
  factory TextSearch.fromJson(final Map<String, dynamic> data) {
    String database;
    String table;
    String text;
    int offset;

    if (data.containsKey('database') && data['database'] is String) {
      database = data['database'] as String;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'database'");
    }

    if (data.containsKey('table') && data['table'] is String) {
      table = data['table'] as String;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'table'");
    }

    if (data.containsKey('text') && data['text'] is String) {
      text = data['text'] as String;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'text'");
    }

    if (data.containsKey('offset') && data['offset'] is int) {
      offset = data['offset'] as int;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'offset'");
    }

    return new TextSearch._from(data, database, table, text, offset);
  }
}

/// Return results of a text search.
class TextSearchResults extends OmnibusResponse {
  /// omnibusId
  static const id = 'molview_messages.v0.TextSearchResults';

  /// N best matches for the given search text
  final List<TextSearchMatch> matches;

  /// Constructor
  TextSearchResults(final List<TextSearchMatch> matches)
      : matches = matches,
        super(id, {'matches': matches});

  /// Internal constructor for fromJson
  TextSearchResults._from(
      final Map<String, dynamic> data, List<TextSearchMatch> matches)
      : matches = matches,
        super(id, data);

  /// Data map input constructor
  factory TextSearchResults.fromJson(final Map<String, dynamic> data) {
    List<TextSearchMatch> matches;

    if (data.containsKey('matches') &&
        data['matches'] is List<Map<String, dynamic>>) {
      matches = new List<TextSearchMatch>.generate(
          data['matches'].length,
          (int i1) => new TextSearchMatch.fromJson(
              data['matches'][i1] as Map<String, dynamic>));
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'matches'");
    }

    return new TextSearchResults._from(data, matches);
  }
}

/// Try to switch to a new active record.
class PushActiveRecord extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_messages.v0.PushActiveRecord';

  /// Target key data class
  final String key;

  /// Target key value (encoded as String)
  final String value;

  /// Constructor
  PushActiveRecord(final String key, final String value)
      : key = key,
        value = value,
        super.unchecked(id, {'key': key, 'value': value});

  /// Internal constructor for fromJson
  PushActiveRecord._from(
      final Map<String, dynamic> data, String key, String value)
      : key = key,
        value = value,
        super.unchecked(id, data);

  /// Data map input constructor
  factory PushActiveRecord.fromJson(final Map<String, dynamic> data) {
    String key;
    String value;

    if (data.containsKey('key') && data['key'] is String) {
      key = data['key'] as String;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'key'");
    }

    if (data.containsKey('value') && data['value'] is String) {
      value = data['value'] as String;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'value'");
    }

    return new PushActiveRecord._from(data, key, value);
  }
}

/// Switch to a new active record.
class ActiveRecordChanged extends OmnibusEvent {
  /// omnibusId
  static const id = 'molview_messages.v0.ActiveRecordChanged';

  /// Target key data class
  final String key;

  /// Target key value (encoded as String)
  final String value;

  /// Constructor
  ActiveRecordChanged(final String key, final String value)
      : key = key,
        value = value,
        super.unchecked(id, {'key': key, 'value': value});

  /// Internal constructor for fromJson
  ActiveRecordChanged._from(
      final Map<String, dynamic> data, String key, String value)
      : key = key,
        value = value,
        super.unchecked(id, data);

  /// Data map input constructor
  factory ActiveRecordChanged.fromJson(final Map<String, dynamic> data) {
    String key;
    String value;

    if (data.containsKey('key') && data['key'] is String) {
      key = data['key'] as String;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'key'");
    }

    if (data.containsKey('value') && data['value'] is String) {
      value = data['value'] as String;
    } else {
      throw new ArgumentError.value(
          data, 'data', "No valid field for property 'value'");
    }

    return new ActiveRecordChanged._from(data, key, value);
  }
}
